/**********************************************************************************************
***********************************************************************************************
Macro Name: MULTIPLE_PHREG
Created Date/Author: Feb. 2012/Yuan Liu
Last Update Date/Person: Feb, 2015/ Yuan Liu
Current Version: V20
Working Environment: SAS 9.3 English version

Contact: Dr. Yuan Liu yliu31@emory.edu

Purpose:  To produce a summary table from a multivariable survival model, but the macro itself 
does not do statistical modeling automatically.  

Notes: The macro only helps to generate a nice summary table, and cannot be used for 
statistical modeling.  The modeling must be conducted using PROC PHREG before calling this 
macro.  Several tables must be created using ODS OUTPUT when running PROC PHREG (nNObs = numobs 
ParameterEstimates = MLEC ModelInfo=modelinf SliceDiffs = slices_diff(only for interaction model)
Type3 = type3). The option RL must also be used in the model statement and option PARAM= GLM in 
class statement. Variable names must not be more than 20 characters.  This macro can be set up to 
handle models including interaction terms, and in the model fitting step, use SLICE statement to specify 
how the stratified analysis to be conducted and use ods output SliceDiffs = slices_diff.   

Parameters: 

OUTPATH   	   Path for output table to be stored.
FNAME     	   File name for output table.
FOOTNOTE  	   Text of footnote to include in the table (optional).  It should be in quotes.  This 
          	   footnote will appear below the footnote containing the number of observations.  
          	   Leave this field blank if not including an additional footnote.
FOOTNOTE2 	   Text of second footnote to include in the table (optional).  It should be in quotes.  
         	   This footnote will appear below all other footnotes.  Leave this field blank if not
           	   including an additional footnote.   
TYPE3     	   Set to F to suppress type III p-values from being reported in the table (optional).
               The default value is T.
EFFECT         If not empty, then the model contains a two-way interaction. It is used to specify
			   the treatment effect in the interaction.
SLICEBY		   Use in combine with EFFECT to specify the stratified variable in the interaction.
SHORTREPORT	   Use in combine with EFFECT and SLICEBY when there is an interaction in the model and set
			   to T to only report the stratified treatment effect.	
ORIENTATION	   orientation of the output Word table. Default is portrait, can be changed to landscape.
DEBUG          Set to T if running in debug mode (optional).  Work datasets will not be deleted in
               debug mode.  This is useful if you are editing the code or want to further 
               manipulate the resulting data sets.  The default value is F.

***********************************************************************************************
For more details, please see the related documentation
**********************************************************************************************/

%macro MULTIPLE_PHREG(OUTPATH=, FNAME =, FOOTNOTE=, FOOTNOTE2=, TYPE3=T, EFFECT=,SLICEBY=,shortreport=T,
ORIENTATION = portrait, DEBUG=F);

   %local FOOTNOTE FOOTNOTE2 NUMREAD NUMUSED OUTPATH DSN EVENT VAR CLASSEXIST N CONTRAST
     EVENTVV CLIST TYPE3 FNAME DEBUG;

   /* Upper case */
   %let type3 = %UPCASE(&type3);
   %let debug = %UPCASE(&debug);

   /* Save current options */
   PROC OPTSAVE out=_options;
   RUN;

   /* Get data set name and outcome name */
   PROC SQL noprint;
      select cvalue into :dsn
      from modelinf
      where description = 'Data Set';
      select cvalue into :event
      from modelinf
      where description = 'Dependent Variable';
   QUIT;

   /* Get outcome label */
   data _NULL_; 
      set &dsn (obs=1); 
      call symput('eventvv', VLABEL(&event));
   run;

      /* Get variable labels */
   PROC CONTENTS DATA = &dsn out=_cont (rename=(name=parameter)) noprint;
   RUN;


   /* Check to see if class variables were used - s/b classval0 variable */
   data _null_;
      dsid=open('mlec');
      check=varnum(dsid,'classval0');
      /* 0 = does not exist, 1 = exists */
      exist = MIN(check, 1);
      CALL SYMPUT('classExist',PUT(exist,1.));
   run;

   /* If class variables were used */
   /* Don't include interaction terms in categorical list */
   %if &classexist = 1 %then %do;
      /* Create list of class variables */
      PROC SQL noprint;
         select distinct Parameter into :clist separated by ' ' 
         from mlec
         where MISSING(classval0) = 0 and INDEX(parameter,'*') = 0 ;
      QUIT;
   %end;
   %else %let clist =;

   DATA mlec;
      set mlec;
      /* Save order */
      Num= _N_;
   RUN;

   %IF &CLIST NE %THEN %DO; 

      data type3;
          length effect $100;
           set type3; 
           order=_n_; 
           rename ProbChiSq = p_type3 effect=parameter; 
           format ProbChiSq;
           keep effect ProbChiSq order;
      run;

      ODS EXCLUDE ALL;
      ods output "One-Way Frequencies" = freq;
      PROC FREQ DATA=&dsn ORDER=internal; 
         TABLE &CLIST;
      RUN;  
      ODS EXCLUDE NONE;

       data freq;
          length Level $256. plabel $256. parameter $100.;
           set freq;
           %LET N = 1; 
           %DO %UNTIL (%SCAN(&CLIST, &N) =   ); 
              %LET VAR = %SCAN(&CLIST, &N); 
              if MISSING(&VAR) = 0 then do;
               Level = left(F_&VAR); 
               pLabel = left(label(&VAR)); 
            end;
               %LET N = %EVAL(&N+1);
           %END;
          parameter = substr(Table, 7,100);
           keep parameter level plabel;
      run;

       proc sort data=freq;by parameter;run;
       proc sort data=type3;by parameter;run;

       data MG; merge type3 freq; by parameter;run; 

       proc sort data=mg;by parameter level;run;
       proc sort data=mlec;by parameter ClassVal0;run;

      data mergedata;
           merge mg mlec(rename=(ClassVal0=Level));
          by parameter level;

           if label ne "" and plabel = "" then plabel = label;
           else if label = "" and plabel = "" then plabel = parameter;
           format ProbChiSq;

           /* If coding was ref then reference parameters will not have been in the original */
         /* parameter estimates file and will have missing num - Make sure they are ordered */
           /* as the last category for a variable */
           if num = . then num = 1000;
      run;

       proc sort data=mergedata;
         by order num;
      run; 

   %END;

   %ELSE %DO;

      /* Check to see if label variable exists (it won't exist if none of the model vars have */
      /* labels */
      data _null_;
         dsid=open('mlec');
         check=varnum(dsid,'label');
           /* 0 = does not exist, 1 = exists */
           exist = MIN(check, 1);
         CALL SYMPUT('labexist',PUT(exist,1.));
      run;

      data mergedata;
         length plabel $256;
         set mlec;
         Level = "";
         p_type3 =     ProbChiSq;

         /* If there are no labels use var names */
         %if &labexist ~= 0 %then %do; 
            if label ~= ' ' then plabel = label; 
               else plabel = parameter;
         %end;
         %else %do;
            plabel = Parameter;
         %end;

         format p_type3 ProbChiSq;
      run;

   %END;

   proc sql noprint;
      select NObsRead into :numread  from numobs; 
      select NObsUsed into :numused from numobs; 
   quit;

   %put &numread;

   /* If reporting contrasts as well */
  %if &effect ~=%str() %then %do;

      	/*Extract variable name,label and length*/

		 DATA _NULL_;
	     set slices_diff (obs=1);
	     call symput('interaction', effect);
	     RUN;

	     DATA _NULL_;
         set &dsn (obs=1);
         call symput('TestVarlab', VLABEL(&effect));
		 call symput('SliceVarlab', VLABEL(&sliceby));
         RUN;	 

		 Data _NULL_;
		 set type3;
		 call symput('testinter',PUT(p_type3,6.4) );
		 where parameter = "&interaction";
		 run;

		 %put &testinter;

		 DATA _cont;
            set _cont end=last;
            if label = ' ' then label = name;
            if _N_ ~= 1 and last then label = 'and ' || label;
         RUN;

		 proc sql;
		 create table _label_ as
		 select a.parameter, b.label, b.parameter
		 from type3 a left join _cont b
		 on a.parameter = b.parameter
		 where a.parameter not in ("&effect"  "&sliceby"  "&interaction");
		 quit;

		 proc sql;
		 select label into :conlab separated by ','
		 from _label_;
		 quit;


      /* Create break row with header */     
      DATA _temp;
          length plabel $256;
          plabel = "Stratified Comparisons by &SliceVarlab (INTERACTION):";
		   p_type3 = &testinter;
       RUN;
       /* Prepare data to be merged on */
	  DATA contrastestimate; set slices_diff;
		plabel = tranwrd(slice,"&sliceby", "");
		level = TRIM(vvalue(&effect)) || " vs. " || TRIM(vvalue(_&effect)) ;	
		HazardRatio = ExpEstimate;
		HRLowerCL = LowerExp;
		HRUpperCL = UpperExp;
		ProbChiSq = Probz;
	  Run;


      DATA _contrast;
         set _temp contrastestimate;
         keep plabel level HazardRatio HRLowerCL HRUpperCL ProbChiSq  p_type3;
      RUN;

	  %if &shortreport = T %then %do;
        Data mergedata;
	      set _contrast;run; %end;
	   %else %do;
	      DATA mergedata;
          set mergedata _contrast; 
		  if parameter not in ("&effect"  "&sliceby"  "&interaction");
          RUN;
	   %end;

   %end;

   DATA mergedata;
      set mergedata;
      /* HR and 95% CI */
      if HazardRatio ~= . then HR = TRIM(LEFT(PUT(HazardRatio,8.2))) || " (" || 
         TRIM(LEFT(PUT(HRLowerCL,8.2))) || "-" || 
         TRIM(LEFT(PUT(HRUpperCL,8.2))) || ")";
      else HR = '-';
      /* Number of observations read and used - save in data set */
      numused = &numused;
      numread = &numread;
   RUN;

   *---- table template -----;  
   ODS PATH WORK.TEMPLAT(UPDATE) SASUSR.TEMPLAT(UPDATE) SASHELP.TMPLMST(READ);

   PROC TEMPLATE;
      DEFINE STYLE STYLES.TABLES;
      NOTES "MY TABLE STYLE"; 
      PARENT=STYLES.MINIMAL;

       STYLE SYSTEMTITLE /FONT_SIZE = 12pt   FONT_FACE = "TIMES NEW ROMAN";

       STYLE HEADER /
           FONT_FACE = "TIMES NEW ROMAN"
            CELLPADDING=8
            JUST=C
            VJUST=C
            FONT_SIZE = 10pt
           FONT_WEIGHT = BOLD; 

     STYLE TABLE /
            FRAME=HSIDES            /* outside borders: void, box, above/below, vsides/hsides, lhs/rhs */
           RULES=GROUP              /* internal borders: none, all, cols, rows, groups */
           CELLPADDING=6            /* the space between table cell contents and the cell border */
            CELLSPACING=6           /* the space between table cells, allows background to show */
            JUST=C
            FONT_SIZE = 10pt
           BORDERWIDTH = 0.5pt;  /* the width of the borders and rules */

     STYLE DATAEMPHASIS /
           FONT_FACE = "TIMES NEW ROMAN"
           FONT_SIZE = 10pt
           FONT_WEIGHT = BOLD;

     STYLE DATA /
           FONT_FACE = "TIMES NEW ROMAN" 
           FONT_SIZE = 10pt;

     STYLE SYSTEMFOOTER /FONT_SIZE = 9pt FONT_FACE = "TIMES NEW ROMAN" JUST=C;
END;

RUN; 



   **** PRINT THE TABLE ************;
   OPTIONS ORIENTATION=&ORIENTATION MISSING = "-" NODATE;
   ODS rtf STYLE=TABLES file= "&OUTPATH.&FNAME &SYSDATE..DOC"; 

   PROC REPORT DATA=mergedata HEADLINE HEADSKIP CENTER STYLE(REPORT)={JUST=CENTER} SPLIT='~' 
     nowd SPANROWS lS=256; 
      COLUMN plabel  Level ("&eventvv" '------------------------------------------'
          (HR  ProbChiSq  %if &type3 = T %then %do; p_type3 %end;)); 
      DEFINE plabel/order order=data "Covariate"  STYLE(COLUMN) = {JUST = L CellWidth=25%}; 
      DEFINE Level/DISPLAY "Level" STYLE(COLUMN) = {JUST =L CellWidth=20%}; 
       %if &type3 = T %then %do; 
         DEFINE p_type3/order MISSING "Type3 P-value" STYLE(COLUMN) = {JUST=C CellWidth=8%} 
               FORMAT=PVALUE8.3; 
       %end;
      DEFINE HR/DISPLAY "Hazard Ratio" STYLE(COLUMN) = {JUST = C CellWidth=15%}; 
      DEFINE ProbChiSq/DISPLAY "HR P-value" STYLE(COLUMN) = {JUST = C CellWidth=8%} FORMAT=PVALUE8.3; 

      COMPUTE ProbChiSq; 
         IF . < ProbChiSq <0.05 THEN CALL DEFINE("ProbChiSq", "STYLE", "STYLE=[FONT_WEIGHT=BOLD]"); 
         if plabel = "Stratified Comparisons by &SliceVarlab (INTERACTION):" THEN CALL DEFINE("ProbChiSq", "STYLE", "STYLE=[COLOR=WHITE]");
      ENDCOMP; 

       COMPUTE plabel;
          if plabel = "Stratified Comparisons by &SliceVarlab (INTERACTION):" THEN CALL DEFINE("plabel", "STYLE", "STYLE=[FONT_WEIGHT=BOLD]");  
       ENDCOMP;

       COMPUTE HR;
          if plabel = "Stratified Comparisons by &SliceVarlab (INTERACTION):" THEN CALL DEFINE("HR", "STYLE", "STYLE=[COLOR=WHITE]");
       ENDCOMP;

       %if &type3 = T %then %do;
         COMPUTE p_type3; 
            IF . < p_type3 <0.05 THEN CALL DEFINE("p_type3", "STYLE", "STYLE=[FONT_WEIGHT=BOLD]"); 
         ENDCOMP;   
       %end;

       compute after plabel; line '';   endcomp;  

       compute after _page_;
         length text1 - text2 $120.;
         text1 = "*  Number of observations in the original data set = %TRIM(&numread). Number of observations used = %TRIM(&numused).";
         line @0 text1 $120.;

           %if &footnote ~= %STR() %then %do;
              line @0 &footnote;
           %end;
           %if &footnote2 ~= %STR() %then %do;
              line @0 &footnote2;
           %end;
		   %if &effect ~=%str() and &shortreport = T %then %do;
			line @0 "*** The estimated stratified treatement effect was controlled by: &conlab";
		   %end;
      ENDCOMP;
 
   RUN;

   ods rtf close;

   /* Reload original options that were in use before running the macro */
   PROC OPTLOAD data=_options;
   RUN;

   %if &debug = F %then %do;
      *--- DELETE ALL TEMPORARY DATASETS that were created; 
      proc datasets lib=work memtype=data noprint;  
          delete mergedata _options %if &CLIST ~= %STR() %then %do; freq mg %end;
               %if &effect = T %then %do; _contrast _temp %end;;
      quit;    
   %end;

%mend;
